package com.qmth.sop.business.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.sop.business.bean.result.DingSubmitResult;
import com.qmth.sop.business.bean.result.DingSubmitSubTotalResult;
import com.qmth.sop.business.entity.TBDingSubmit;
import com.qmth.sop.business.mapper.TBDingSubmitMapper;
import com.qmth.sop.business.service.TBDingSubmitService;
import com.qmth.sop.common.contant.SystemConstant;
import com.qmth.sop.common.enums.DingSubmitStatusEnum;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Description: 考勤提交服务实现类
 * @Author: CaoZixuan
 * @Date: 2023/08/25
 */
@Service
public class TBDingSubmitServiceImpl extends ServiceImpl<TBDingSubmitMapper, TBDingSubmit> implements TBDingSubmitService {
    @Override
    public IPage<DingSubmitResult> findDingSubmitPage(Long serviceUnitId, String dingSubmitStatusDesc, Long submitUserId,
                                                      String archivesName, Long supplierId, String customName, String sopNo,
                                                      Integer violationHoursLimit, Integer dingExceptionLimit, Integer remainLimit, Integer exceptionLimit,
                                                      Integer pageNumber, Integer pageSize) {

        List<DingSubmitStatusEnum> dingSubmitStatusEnumList = new ArrayList<>();
        if (dingSubmitStatusDesc != null && dingSubmitStatusDesc.length() > 0) {
            dingSubmitStatusEnumList = Arrays.stream(DingSubmitStatusEnum.values())
                    .filter(e -> e.getSubmitDesc().equals(dingSubmitStatusDesc))
                    .collect(Collectors.toList());
        }
        archivesName = SystemConstant.translateSpecificSign(archivesName);
        customName = SystemConstant.translateSpecificSign(customName);
        return this.baseMapper.findDingSubmitPage(new Page<>(pageNumber, pageSize), serviceUnitId, dingSubmitStatusEnumList,
                submitUserId, archivesName, supplierId, customName, sopNo, violationHoursLimit, dingExceptionLimit, remainLimit, exceptionLimit);
    }

    @Override
    public DingSubmitSubTotalResult findDingSubmitSubTotal(Long serviceUnitId, String dingSubmitStatusDesc, Long submitUserId,
                                                           String archivesName, Long supplierId, String customName, String sopNo,
                                                           Integer violationHoursLimit, Integer dingExceptionLimit,
                                                           Integer remainLimit, Integer exceptionLimit) {
        List<DingSubmitStatusEnum> dingSubmitStatusEnumList = new ArrayList<>();
        if (dingSubmitStatusDesc != null && dingSubmitStatusDesc.length() > 0) {
            dingSubmitStatusEnumList = Arrays.stream(DingSubmitStatusEnum.values())
                    .filter(e -> e.getSubmitDesc().equals(dingSubmitStatusDesc))
                    .collect(Collectors.toList());
        }
        archivesName = SystemConstant.translateSpecificSign(archivesName);
        customName = SystemConstant.translateSpecificSign(customName);
        List<DingSubmitResult> dingSubmitResultList = this.baseMapper.findDingSubmitList(serviceUnitId, dingSubmitStatusEnumList,
                submitUserId, archivesName, supplierId, customName, sopNo, violationHoursLimit, dingExceptionLimit, remainLimit, exceptionLimit);

        Integer dingCount = dingSubmitResultList.size();
        Integer willSubmitCount = Math.toIntExact(dingSubmitResultList.stream().filter(e -> e.getSubmitStatus().getSubmitDesc().equals("待提交")).count());
        Integer submitCount = Math.toIntExact(dingSubmitResultList.stream().filter(e -> e.getSubmitStatus().getSubmitDesc().equals("已提交")).count());
        Integer totalActualDays = dingSubmitResultList.stream().mapToInt(DingSubmitResult::getActualDays).sum();
        Double totalWorkHours = dingSubmitResultList.stream().mapToDouble(DingSubmitResult::getWorkHours).sum();
        DingSubmitSubTotalResult dingSubmitSubTotalResult = new DingSubmitSubTotalResult();
        dingSubmitSubTotalResult.setDingCount(dingCount);
        dingSubmitSubTotalResult.setWillSubmitCount(willSubmitCount);
        dingSubmitSubTotalResult.setSubmitCount(submitCount);
        dingSubmitSubTotalResult.setTotalActualDays(totalActualDays);
        dingSubmitSubTotalResult.setTotalWorkHours(totalWorkHours);
        return dingSubmitSubTotalResult;
    }

    @Override
    public void submitDingBatch() {

    }

    @Override
    public void applyWithDraw(Long archivesId, String crmNo) {

    }

    @Override
    public void cancelWithDraw(Long archivesId, String crmNo) {

    }
}
